#!/usr/bin/python3

import os
import unittest

import unattended_upgrade
from test.test_base import TestBase, MockOptions


class TestRewindCache(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.mock_allowed_origins("origin=Ubuntu,archive=lucid-security")
        rootdir = self.make_fake_aptroot(os.path.join(self.testdir, "root.rewind"))
        self.cache = unattended_upgrade.UnattendedUpgradesCache(rootdir=rootdir)

    def test_rewind_cache(self):
        """ Test that rewinding the cache works correctly, debian #743594 """
        options = MockOptions()
        options.try_run = True
        to_upgrade = unattended_upgrade.calculate_upgradable_pkgs(self.cache, options)
        self.assertEqual(to_upgrade, [self.cache[p] for p
                                      in ["test-package", "test2-package",
                                          "test3-package"]])
        unattended_upgrade.rewind_cache(self.cache, to_upgrade)
        self.assertEqual(self.cache['test-package'].candidate.version, "2.0")


if __name__ == "__main__":
    import logging
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3

import os
import unittest

from test.test_base import TestBase, MockOptions

import unattended_upgrade


class TestUntrusted(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.rootdir = self.make_fake_aptroot(
            template=os.path.join(self.testdir, "root.untrusted"))
        self.log = os.path.join(
            self.rootdir, "var", "log", "unattended-upgrades",
            "unattended-upgrades.log")

    def test_untrusted_check_without_conffile_check(self):
        options = MockOptions()
        unattended_upgrade.main(options, rootdir=self.rootdir)
        # read the log to see what happend
        with open(self.log) as f:
            haystack = f.read()
            self.assertTrue(
                "pkg test-package is not from a trusted origin" in haystack)
            if unattended_upgrade.get_distro_codename() == "sid":
                self.assertFalse(
                    "falling back to adjusting" in haystack)
            else:
                self.assertTrue(
                    "falling back to adjusting" in haystack)


if __name__ == "__main__":
    # do not setup logging in here or the test will break
    unittest.main()



#!/usr/bin/python3

import logging
import unittest

import apt_pkg


from unattended_upgrade import substitute, get_allowed_origins

from test.test_base import TestBase


class TestSubstitute(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.mock_distro("MyDistroID", "mycodename", "MyDistroID descr")

    def testSubstitute(self):
        """ test if the substitute function works """
        self.assertTrue(substitute("${distro_codename}-updates"),
                        "nacked-updates")
        self.assertTrue(substitute("${distro_id}"), "MyDistroID")

    def test_get_allowed_origins_with_substitute(self):
        """ test if substitute for get_allowed_origins works """
        apt_pkg.config.clear("Unattended-Upgrade::Allowed-Origins")
        apt_pkg.config.set("Unattended-Upgrade::Allowed-Origins::",
                           "${distro_id} ${distro_codename}-security")
        li = get_allowed_origins()
        self.assertIn("o=MyDistroID,a=mycodename-security", li)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3

import json
import os
import shutil

from unittest.mock import patch

import unattended_upgrade
from test.test_base import TestBase, MockOptions


class TestPlugins(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        # XXX: copy/clean to root.plugins
        self.rootdir = self.make_fake_aptroot(
            template=os.path.join(self.testdir, "root.unused-deps"),
            fake_pkgs=[("test-package", "1.0.test.pkg", {})],
        )
        # create a fake plugin dir, our fake plugin just writes a json
        # dump of it's args
        fake_plugin_dir = os.path.join(self.tempdir, "plugins")
        os.makedirs(fake_plugin_dir, 0o755)
        example_plugin = os.path.join(
            os.path.dirname(__file__), "../examples/plugins/simple.py")
        shutil.copy(example_plugin, fake_plugin_dir)
        os.environ["UNATTENDED_UPGRADES_PLUGIN_PATH"] = fake_plugin_dir
        self.addCleanup(lambda: os.environ.pop("UNATTENDED_UPGRADES_PLUGIN_PATH"))
        # go to a tempdir as the simple.py plugin above will just write a
        # file into cwd
        os.chdir(self.tempdir)

    @patch("unattended_upgrade.reboot_required")
    @patch("unattended_upgrade.host")
    def test_plugin_happy(self, mock_host, mock_reboot_required):
        mock_reboot_required.return_value = True
        mock_host.return_value = "some-host"
        options = MockOptions()
        options.debug = False
        unattended_upgrade.main(options, rootdir=self.rootdir)
        with open("simple-example-postrun-res.json") as fp:
            arg1 = json.load(fp)
        # the log text needs extra processing
        log_dpkg = arg1.pop("log_dpkg")
        log_uu = arg1.pop("log_unattended_upgrades")
        self.assertEqual(arg1, {
            "plugin_api": "1.0",
            "hostname": "some-host",
            "success": True,
            "result": "All upgrades installed",
            "packages_upgraded": ["test-package"],
            # XXX: add test for pkgs_kept_back
            "packages_kept_back": [],
            # XXX: add test for pkgs_kept_installed
            "packages_kept_installed": [],
            # XXX: add test for pkgs_removed
            "packages_removed": [],
            "reboot_required": True,
        })
        self.assertTrue(log_dpkg.startswith("Log started:"))
        self.assertTrue(
            "Starting unattended upgrades script" in log_uu)
        self.assertIn("Packages that will be upgraded: test-package", log_uu)

    def test_plugin_data_postrun(self):
        res = unattended_upgrade.PluginDataPostrun({
            "plugin_api": "1.0",
            "hostname": "some-host",
            "success": True,
            "result": "some result str",
            "packages_upgraded": ["upgrade-pkg1", "upgrade-pkg2"],
            "packages_kept_back": ["kept-pkg1"],
            "packages_removed": ["rm-pkg1"],
            "packages_kept_installed": ["kept-installed-pkg1"],
            "reboot_required": False,
            "log_dpkg": "a very long dpkg log",
        })
        # ensure properties keep working
        self.assertEqual(res.plugin_api, "1.0")
        self.assertEqual(res.hostname, "some-host")
        self.assertEqual(res.success, True)
        self.assertEqual(res.result, "some result str")
        self.assertEqual(
            res.packages_upgraded, ["upgrade-pkg1", "upgrade-pkg2"])
        self.assertEqual(res.packages_kept_back, ["kept-pkg1"])
        self.assertEqual(res.packages_removed, ["rm-pkg1"])
        self.assertEqual(res.packages_kept_installed, ["kept-installed-pkg1"])
        self.assertEqual(res.reboot_required, False)
        self.assertEqual(res.log_dpkg, "a very long dpkg log")



#!/usr/bin/python3

import logging
import os
import os.path
import shutil
import subprocess
import tempfile
import unittest

from unittest.mock import patch

import apt

import unattended_upgrade


class MockOptions(object):
    debug = True
    verbose = False
    download_only = False
    dry_run = False
    apt_debug = False
    minimal_upgrade_steps = True


class TestBase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        pkgdir = os.path.join(os.path.dirname(__file__), "packages")
        subprocess.check_call(["make", "-C", pkgdir])

    def setUp(self):
        super(TestBase, self).setUp()
        self.tempdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tempdir)
        self.testdir = os.path.dirname(__file__)
        # ensure custom logging gets reset (XXX: does this work?)
        self.addCleanup(logging.shutdown)
        logging.root.handlers = []
        # XXX: workaround for most tests assuming to run inside the "test"
        # dir
        os.chdir(self.testdir)
        # fake the lock file
        unattended_upgrade.LOCK_FILE = os.path.join(self.tempdir, "u-u.lock")
        # reset apt config
        apt.apt_pkg.init_config()
        apt.apt_pkg.config.set("APT::Architecture", "amd64")
        # must be last
        self._saved_apt_conf = {}
        for k in apt.apt_pkg.config.keys():
            if not k.endswith("::"):
                self._saved_apt_conf[k] = apt.apt_pkg.config.get(k)
        self.addCleanup(self.enforce_apt_config_reset)
        # important to ensure that the updated apt config is applied
        self.addCleanup(apt.apt_pkg.init_system)

    def enforce_apt_config_reset(self):
        for k in self._saved_apt_conf:
            v = self._saved_apt_conf[k]
            apt.apt_pkg.config.set(k, v)

    def make_fake_aptroot(self, template=None, fake_pkgs=[]):
        tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, tmpdir)
        aptroot = os.path.join(tmpdir, "aptroot")
        shutil.copytree(template, aptroot)
        # apt warns if this dir does not exist
        os.makedirs(os.path.join(aptroot, "etc/apt/preferences.d"))
        # fake dpkg status
        mock_dpkg_status = os.path.join(aptroot, "var/lib/dpkg/status")
        with open(mock_dpkg_status, "a") as fp:
            for (pkgname, ver, deps) in fake_pkgs:
                # only "Depends" supported right now, extend as needed
                self.assertIn(set(deps.keys()), (set([]), set(["Depends"])))
                dep_str = ""
                if deps.get("Depends"):
                    dep_str = "Depends: {}".format(",".join(deps.get("Depends", [])))
                fp.write(
                    """
Package: %s
Status: install ok installed
Architecture: all
Version: %s
%s
"""
                    % (pkgname, ver, dep_str)
                )
        # setup everything to run dpkg
        apt.apt_pkg.config.set("Dir::State::status", mock_dpkg_status)
        apt.apt_pkg.config.clear("DPkg::Pre-Invoke")
        apt.apt_pkg.config.clear("DPkg::Post-Invoke")
        apt.apt_pkg.config.clear("DPkg::Pre-Install-Pkgs")
        apt.apt_pkg.config.set("Debug::NoLocking", "true")
        # we don't really run dpkg
        fake_dpkg = os.path.join(aptroot, "usr", "bin", "dpkg")
        if not os.path.exists(fake_dpkg):
            os.makedirs(os.path.dirname(fake_dpkg), exist_ok=True)
            with open(fake_dpkg, "w") as fp:
                fp.write(
                    """#!/usr/bin/python3
import sys
print(sys.argv)
"""
                )
        os.chmod(fake_dpkg, 0o755)
        apt.apt_pkg.config.set("Dir::Bin::Dpkg", fake_dpkg)
        return aptroot

    def mock_distro(self, distro_id, codename, descr):
        for attr, fake_value in [
            ("DISTRO_ID", distro_id),
            ("DISTRO_CODENAME", codename),
            ("DISTRO_DESC", descr),
        ]:
            patcher = patch("unattended_upgrade.{}".format(attr), fake_value)
            patcher.start()
            self.addCleanup(patcher.stop)

    def mock_allowed_origins(self, origin_pattern):
        apt.apt_pkg.config.set(
            "Unattended-Upgrade::Origins-Pattern::", origin_pattern)
        self.addCleanup(apt.apt_pkg.config.clear, "Unattended-Upgrade::Origins-Pattern")



#!/usr/bin/python3

import logging
import os
import unittest

import apt_pkg


from unattended_upgrade import _setup_logging
from test.test_base import TestBase, MockOptions


class TestLogdir(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.mock_options = MockOptions()

    def test_logdir(self):
        # test log
        logdir = os.path.join(self.tempdir, "mylog")
        apt_pkg.config.set("Unattended-Upgrade::LogDir", logdir)
        _setup_logging(self.mock_options)
        self.assertTrue(os.path.exists(logdir))

    def test_logdir_depreated(self):
        # test if the deprecated APT::UnattendedUpgrades dir is not used
        # if the new UnaUnattendedUpgrades::LogDir is given
        logdir = os.path.join(self.tempdir, "mylog-use")
        logdir2 = os.path.join(self.tempdir, "mylog-dontuse")
        apt_pkg.config.set("Unattended-Upgrade::LogDir", logdir)
        apt_pkg.config.set("APT::UnattendedUpgrades::LogDir", logdir2)
        logging.root.handlers = []
        _setup_logging(self.mock_options)
        self.assertTrue(os.path.exists(logdir))
        self.assertFalse(os.path.exists(logdir2))


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main()
